﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.SharePoint;
using RH.NoBlogCommentSpam;
using Microsoft.SharePoint.Administration;

namespace RH
{
    public class Common
    {
        /// <summary>
        /// Check if the list exists in the web
        /// </summary>
        /// <param name="web"></param>
        /// <param name="listId">Guid</param>
        /// <returns></returns>
        internal static SPList GetList(SPWeb web, Guid listId)
        {
            try
            {
                SPList list = web.Lists.GetList(listId, true);
                return list;
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetList' for Web '{0}' and ListId '{1}': {2}", web.Url, listId, ex));
                return null;
            }
        }

        /// <summary>
        /// checks if the feature is activated for the current spweb
        /// </summary>
        internal static bool FeatureActivated(SPWeb web, Guid featureId)
        {
            foreach (SPFeature feature in web.Features)
            {
                if (feature.DefinitionId == featureId)
                    return true;
            }

            return false;
        }

        public static IList<NoBlogCommentProperty> GetProperties(string jobProperties)
        {
            IList<NoBlogCommentProperty> properties = new List<NoBlogCommentProperty>();
            try
            {
                if (jobProperties != null)
                {
                    // generate objects from string which was saved to job properties
                    foreach (string jobProperty in jobProperties.Split('|'))
                        properties.Add(new NoBlogCommentProperty(jobProperty));
                }
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetProperties' with jobProperties '{0}': {1}", jobProperties, ex));
            }

            return properties;
        }

        /// <summary>
        /// find properties, where the url startsWith "webUrl"
        /// </summary>
        /// <param name="properties"></param>
        /// <param name="webUrl"></param>
        /// <param name="jobTitle"></param>
        /// <returns></returns>
        internal static IList<NoBlogCommentProperty> GetProperties(IList<NoBlogCommentProperty> properties, string webUrl, string jobTitle)
        {
            try
            {
                var myProperties = properties.Where(p => p.WebUrl.StartsWith(webUrl)) ?? new List<NoBlogCommentProperty>();

                return myProperties.ToArray();
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetProperties' for Web '{0}': {1}", webUrl, ex));
            }
            
            return new List<NoBlogCommentProperty>();
        }

        /// <summary>
        /// find a property
        /// </summary>
        /// <returns></returns>
        internal static NoBlogCommentProperty GetProperty(IList<NoBlogCommentProperty> properties, SPList list, string webUrl)
        {
            try
            {
                if (properties.Count == 1 && properties[0] == null)
                {
                    // in case we have an empty property
                    properties[0] = new NoBlogCommentProperty
                                        {
                                            WebUrl = webUrl,
                                            ListId = list.ID,
                                            ApproveNonAnonymousComments = false,
                                            TitleRecycle = false,
                                            BodyRecycle = false,
                                            UrlMatch = false
                                        };
                    return properties[0];
                }

                // find propertiy from collection
                var property = properties.Where(p => p.ListId == list.ID).FirstOrDefault();
                if (property == null)
                {
                    // add new property, if there was none
                    property = new NoBlogCommentProperty
                                   {
                                       WebUrl = webUrl,
                                       ListId = list.ID,
                                       ApproveNonAnonymousComments = false,
                                       TitleRecycle = false,
                                       BodyRecycle = false,
                                       UrlMatch = false
                                   };
                    properties.Add(property);
                }
                return property;
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetProperty' for list '{0}' on Web '{1}': {2}", list.Title, webUrl, ex));
            }
            return null;
        }

        internal static bool ContainsJob(SPJobDefinitionCollection jobs, string jobTitle)
        {
            foreach (var job in jobs)
            {
                if (job.Title == jobTitle)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// find job from webapplication
        /// </summary>
        /// <param name="webUrl"></param>
        /// <param name="jobTitle"></param>
        /// <returns></returns>
        internal static SPJobDefinition GetJob(string webUrl, string jobTitle)
        {
            try
            {
                using (var site = new SPSite(webUrl))
                {
                    SPWebApplication webApplication = site.WebApplication;
                    foreach (SPJobDefinition jobDefinition in webApplication.JobDefinitions)
                    {
                        if (jobDefinition.Title == jobTitle)
                            return jobDefinition;
                    }
                }
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetJob' for Web '{0}': {1}", webUrl, ex));
            }
            return null;
        }

        /// <summary>
        /// serialize IList
        /// </summary>
        /// <param name="properties"></param>
        /// <returns></returns>
        public static string GetString(IList<NoBlogCommentProperty> properties)
        {
            string str = string.Empty;
            try
            {
                foreach (NoBlogCommentProperty property in properties)
                    str += property + "|";

                if (str.EndsWith("|"))
                    str = str.Substring(0, str.Length - 1);
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetString': {0}", ex));
            }
            return str;
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Exception)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteULSError(string text)
        {
            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Exception, Guid.Empty, "RH.NoBlogCommentSpam.dll", "RH.NoBlogCommentSpam", "General", text);
                TraceProvider.UnregisterTraceProvider();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Verbose)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteULSVerbose(string text)
        {
            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Verbose, Guid.Empty, "RH.NoBlogCommentSpam.dll", "RH.NoBlogCommentSpam", "General", text);
                TraceProvider.UnregisterTraceProvider();
            }
            catch (Exception)
            {
            }

        }

        /// <summary>
        /// remove a property from the properties of a job PropertyBag
        /// </summary>
        /// <param name="job"></param>
        /// <param name="jobTitle"></param>
        /// <param name="properties"></param>
        /// <param name="property"></param>
        internal static void RemoveProperty(SPJobDefinition job, string jobTitle, IList<NoBlogCommentProperty> properties, NoBlogCommentProperty property)
        {
            try
            {
                WriteULSVerbose(string.Format("Removing Property ({0}) from List '{1}' in Web'{2}'", property, property.ListId, property.WebUrl));
                properties.Remove(property);
                job.Properties[jobTitle] = GetString(properties);
                job.Update();
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Could not remove Property from Job: {0}", ex));            
            }
        }

        /// <summary>
        /// find account from WSS Administration Service
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        internal static string GetServiceName(string url)
        {
            string account = null;
            try
            {
                SPSecurity.RunWithElevatedPrivileges(delegate
                {
                    using (var site = new SPSite(url))
                    using (var rootWeb = site.RootWeb)
                    {
                        SPWebApplication webApplication = site.WebApplication;
                        SPFarm farm = webApplication.Farm;
                        SPServiceCollection services = farm.Services;

                        foreach (SPService service in services)
                        {
                            if (service.Name != "WSS_Administration") continue;

                            var identityCollection = new SPProcessIdentityCollection(service);
                            foreach (SPProcessIdentity identity in identityCollection)
                                account = identity.Username;
                        }
                        // ensure that the user exists in the rootWeb
                        rootWeb.EnsureUser(account);
                    }
                });
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error getting ServiceName: {0}", ex));
            }
            return account;
        }
    }
}
