﻿using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace RH.NoBlogCommentSpam
{
    public class NoBlogCommentSpam : SPJobDefinition
    {
        private const string _JobTitle = "NoBlogCommentSpam";
        private Regex urlRegex;

        public NoBlogCommentSpam()
        {
        }

        public NoBlogCommentSpam(string MyJobName, SPWebApplication MyWebApplication)
            : base(MyJobName, MyWebApplication, null, SPJobLockType.Job)
        {
        }

        public override void Execute(Guid contentDbId)
        {
            try
            {
                urlRegex = new Regex(@"(?:http|https|ftp)://[a-zA-Z0-9\.\-]+(?:\:\d{1,5})?(?:[A-Za-z0-9\.\;\:\@\&\=\+\$\,\?/]|%u[0-9A-Fa-f]{4}|%[0-9A-Fa-f]{2})*", RegexOptions.Multiline);
                var webApplication = Parent as SPWebApplication;
                if (webApplication == null) return;

                // go through each webapplication, and look for an activated feature
                var service = webApplication.Farm.Services.GetValue<SPWebService>("");
                foreach (SPWebApplication application in service.WebApplications)
                {
                    if (application.Features[new Guid("EDC7FD11-75E1-4d81-88FD-A48101F14758")] == null) continue;

                    // find job
                    SPJobDefinition job = null;
                    foreach (SPJobDefinition jobDefinition in application.JobDefinitions)
                    {
                        if (jobDefinition.Title != _JobTitle) continue;
                        job = jobDefinition;
                        break;
                    }
                    if (job == null) continue;

                    var jobProperties = job.Properties[_JobTitle] as string;
                    IList<NoBlogCommentProperty> properties = Common.GetProperties(jobProperties);

                    for (int p = properties.Count-1; p >= 0; p--)
                    {
                        NoBlogCommentProperty property = properties[p];
                        Common.WriteULSVerbose(string.Format("Found Property {0}", property));
                        if (property == null)
                        {
                            Common.RemoveProperty(job, _JobTitle, properties, property);
                            continue;
                        }

                        try
                        {
                            // get the list
                            using (var site = new SPSite(property.WebUrl))
                            using (SPWeb web = site.OpenWeb())
                            {
                                SPList list = Common.GetList(web, property.ListId);
                                if (list == null)
                                {
                                    Common.RemoveProperty(job, _JobTitle, properties, property);
                                    continue;
                                }

                                SPListItemCollection pendingItems = GetPendingListItems(list, web.CurrencyLocaleID);
                                if (pendingItems.Count == 0) Common.WriteULSVerbose(string.Format("List '{0}' has no pending comments", property.ListId));
                                for (int i = pendingItems.Count - 1; i >= 0; i--)
                                {
                                    Common.WriteULSVerbose(string.Format("Working on Item '{0}' (List '{1}', Web '{2}')", pendingItems[i].Title, list.Title, web.Url));
                                    CheckListItemForSpam(pendingItems[i], property);
                                }
                            }
                        }
                        catch (SPException ex)
                        {
                            if (ex.ErrorCode == -2147024894)// SiteUrl is wrong
                                Common.RemoveProperty(job, _JobTitle, properties, property);
                        }
                        catch (Exception ex)
                        {
                            if (ex.Message.StartsWith("Invalid URI:"))
                                Common.RemoveProperty(job, _JobTitle, properties, property);
                            else
                                Common.WriteULSError(string.Format("Error processing ListId '{0}' in Web '{1}': {2}", property.ListId, property.WebUrl, ex));
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Common.WriteULSError(string.Format("Error in 'Execute': {0}", ex));
            }
        }

        /// <summary>
        /// get items from a list, where the moderationstatus is "Pending"
        /// </summary>
        /// <param name="list"></param>
        /// <param name="localeId"></param>
        /// <returns></returns>
        public SPListItemCollection GetPendingListItems(SPList list, int localeId)
        {
            SPListItemCollection results = null;
            try
            {
                var query = new SPQuery { Query = "<Where><Eq><FieldRef Name='_ModerationStatus'/><Value Type='Number'>2</Value></Eq></Where>" };
                results = list.GetItems(query);
            }
            catch (Exception ex)
            {
                Common.WriteULSError(string.Format("Error in 'GetPendingListItems': {0}", ex));
            }
            return results;
        }

        /// <summary>
        /// check if the item contains spam, and approve it
        /// </summary>
        /// <param name="pendingItem"></param>
        /// <param name="property"></param>
        private void CheckListItemForSpam(SPListItem pendingItem, NoBlogCommentProperty property)
        {
            try
            {
                if (Convert.ToString(pendingItem["Author"]) != "-1;#" && property.ApproveNonAnonymousComments)
                {
                    // approve the item, since it was posted by a non anonymous user
                    Common.WriteULSVerbose(string.Format("Approving comment '{0}'", pendingItem.Title));
                    pendingItem.ModerationInformation.Status = SPModerationStatusType.Approved;
                    pendingItem.SystemUpdate();
                    return;
                }

                MatchCollection titleUrls = urlRegex.Matches(Convert.ToString(pendingItem[SPBuiltInFieldId.Title]));
                MatchCollection bodyUrls = urlRegex.Matches(Convert.ToString(pendingItem[SPBuiltInFieldId.Body]));
                if (property.TitleRecycle)
                {
                    // title contains at least one url. recycle it
                    if (titleUrls.Count > 0)
                    {
                        Common.WriteULSVerbose(string.Format("Recycling comment '{0}'", pendingItem.Title));
                        pendingItem.Recycle();
                        return;
                    }
                }

                if (property.BodyRecycle)
                {
                    // body contains at least one url. recycle it
                    if (bodyUrls.Count > 0)
                    {
                        Common.WriteULSVerbose(string.Format("Recycling comment '{0}'", pendingItem.Title));
                        pendingItem.Recycle();
                        return;
                    }
                }

                if (property.UrlMatch)
                {
                    foreach (Match titleUrl in titleUrls)
                        foreach (Match bodyUrl in bodyUrls)
                            if (titleUrl.Value == bodyUrl.Value)
                            {
                                // found a url, which is in the title and body
                                Common.WriteULSVerbose(string.Format("Recycling comment '{0}'", pendingItem.Title));
                                pendingItem.Recycle();
                                return;
                            }
                }

                // if we reached this point, we can approve the comment
                Common.WriteULSVerbose(string.Format("Approving comment '{0}'", pendingItem.Title));
                pendingItem.ModerationInformation.Status = SPModerationStatusType.Approved;
                pendingItem.SystemUpdate();
            }
            catch (Exception ex)
            {
                Common.WriteULSError(string.Format("Error in 'CheckListItemForSpam': {0}", ex));
            }
        }
    }
}