﻿using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using Microsoft.SharePoint.Administration;

namespace RH.NoBlogCommentSpam
{
    public class RHNoBlogCommentSpamSettingsPage : LayoutsPageBase
    {
        protected Label LabelErrorMessage;
        protected Button BtnSave;
        protected CheckBox ApproveNonAnonymous;
        protected CheckBox TitleRecycle;
        protected CheckBox BodyRecycle;
        protected CheckBox UrlMatch;
        protected InputFormSection nonAnonymousComments;
        protected InputFormSection LinkSpamProtection;
        protected SPList List;
        private readonly Guid _FeatureID = new Guid("4CD58D7F-45AD-4b91-840E-EBCC36DC33DA");
        private const string _JobTitle = "NoBlogCommentSpam";

        protected void Page_Load(Object sender, EventArgs e)
        {
            string[] query = Request.QueryString.GetValues("list");
            #region verify query string
            if (query == null || query.Length < 1)
            {
                ShowError("Please call this page from the Settings Menu within a list.");
                nonAnonymousComments.Visible = false;
                LinkSpamProtection.Visible = false;
                return;
            }
            #endregion

            try
            {
                List = Common.GetList(SPContext.Current.Web, new Guid(query[0]));

                #region check if user has rights
                if (!List.DoesUserHavePermissions(SPBasePermissions.ManageLists))
                {
                    ShowError("You do not have the right to manage lists.");
                    nonAnonymousComments.Visible = false;
                    LinkSpamProtection.Visible = false;
                    return;
                }
                #endregion

                if (List != null)
                {
                    #region check if feature is activated for this spweb
                    if (!Common.FeatureActivated(SPContext.Current.Web, _FeatureID))
                    {
                        ShowError("The feature \"RH.NoBlogCommentSpam\" is not activated for the current web.");
                        nonAnonymousComments.Visible = false;
                        LinkSpamProtection.Visible = false;
                        return;
                    }
                    #endregion

                    if (!Page.IsPostBack)
                    {
                        string webUrl = SPContext.Current.Web.Url;
                        SPSecurity.RunWithElevatedPrivileges(delegate
                        {
                            // get data from job to fill controls
                            SPJobDefinition job = Common.GetJob(webUrl, _JobTitle);
                            var jobProperties = job.Properties[_JobTitle] as string;
                            IList<NoBlogCommentProperty> properties = Common.GetProperties(jobProperties);
                            NoBlogCommentProperty property = Common.GetProperty(properties, List, webUrl);
                            Common.WriteULSVerbose(string.Format("Property ({0}) loaded from List '{1}' in Web'{2}'", property, property.ListId, property.WebUrl));
                            ApproveNonAnonymous.Checked = property.ApproveNonAnonymousComments;
                            TitleRecycle.Checked = property.TitleRecycle;
                            BodyRecycle.Checked = property.BodyRecycle;
                            UrlMatch.Checked = property.UrlMatch;
                        });
                    }
                }
            }
            catch (Exception ex)
            {
                throw new SPException(ex.ToString());
            }
        }

        public void BtnSave_Click(Object sender, EventArgs e)
        {
            try
            {
                string webUrl = SPContext.Current.Web.Url;
                string account = Common.GetServiceName(webUrl);
                var systemUser = GetUser(account) ?? GetUser(@"SHAREPOINT\SYSTEM");
                var systemToken = systemUser.UserToken;

                using (var site = new SPSite(webUrl, systemToken))
                {
                    var webApplication = site.WebApplication;
                    var farm = webApplication.Farm;
                    bool admin = farm.CurrentUserIsAdministrator();
                    Common.WriteULSVerbose(string.Format("User '{0}' is farm admin: {1}", systemUser.LoginName, admin));
                    SPJobDefinition job = null;
                    foreach (SPJobDefinition jobDefinition in webApplication.JobDefinitions)
                    {
                        if (jobDefinition.Title == _JobTitle)
                        {
                            job = jobDefinition;
                            break;
                        }
                    }
                    if (job == null) return;

                    var jobProperties = job.Properties[_JobTitle] as string;
                    IList<NoBlogCommentProperty> properties = Common.GetProperties(jobProperties);

                    if (!ApproveNonAnonymous.Checked && !TitleRecycle.Checked && !BodyRecycle.Checked && !UrlMatch.Checked)
                    {
                        // remove property, since nothing to do
                        NoBlogCommentProperty property = Common.GetProperty(properties, List, webUrl);
                        Common.RemoveProperty(job, _JobTitle, properties, property);
                    }
                    else
                    {
                        // add property, so that the timer job handles this list
                        // find "our" property and modify it (or create it, if it does not exist)
                        var property = Common.GetProperty(properties, List, webUrl);

                        property.WebUrl = SPContext.Current.Web.Url;
                        property.ListId = List.ID;
                        property.ApproveNonAnonymousComments = ApproveNonAnonymous.Checked;
                        property.TitleRecycle = TitleRecycle.Checked;
                        property.BodyRecycle = BodyRecycle.Checked;
                        property.UrlMatch = UrlMatch.Checked;

                        Common.WriteULSVerbose(string.Format("Adding Property ({0}) from List '{1}' in Web'{2}'", property, property.ListId, property.WebUrl));
                        job.Properties[_JobTitle] = Common.GetString(properties);
                        job.Update();
                    }
                }

                // redirect
                if (List != null) Response.Redirect(SPContext.Current.Site.MakeFullUrl(List.DefaultViewUrl));
                else Response.Redirect(SPContext.Current.Web.Url);
            }
            catch (Exception ex)
            {
                throw new SPException(ex.ToString());
            }
        }

        /// <summary>
        /// shows an error and hides people fields an submit button
        /// </summary>
        /// <param name="errorMessage"></param>
        internal void ShowError(string errorMessage)
        {
            LabelErrorMessage.Text = string.Format("<strong>{0}</strong><br/><br/>", errorMessage);
            ApproveNonAnonymous.Visible = false;
            BtnSave.Visible = false;
        }

        private SPUser GetUser(string loginName)
        {
            var webUrl = SPContext.Current.Site.Url;
            SPUser user = null;
            try
            {
                SPSecurity.RunWithElevatedPrivileges(delegate
                                                         {
                                                             using (var site = new SPSite(webUrl))
                                                             using (var rootWeb = site.RootWeb)
                                                                 user = rootWeb.AllUsers[loginName];
                                                         });
            }
            catch (Exception ex)
            {
                Common.WriteULSError(string.Format("Error getting the account '{0}': {1}", loginName, ex));
            }
            return user;
        }
    }
}
